﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Activation;
using Windows.Data.Json;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using System.Runtime.Serialization.Json;

// The Blank Application template is documented at http://go.microsoft.com/fwlink/?LinkId=234227

namespace WAZSMApiClient
{
    /// <summary>
    /// Provides application-specific behavior to supplement the default Application class.
    /// </summary>
    sealed partial class App : Application
    {
        public static List<SavedSubscription> SavedSubscriptions;
        /// <summary>
        /// Initializes the singleton application object.  This is the first line of authored code
        /// executed, and as such is the logical equivalent of main() or WinMain().
        /// </summary>
        public App()
        {
            this.InitializeComponent();
            this.Suspending += OnSuspending;
            SavedSubscriptions = new List<SavedSubscription>();
            ReadSavedData();
        }

        /// <summary>
        /// Invoked when the application is launched normally by the end user.  Other entry points
        /// will be used when the application is launched to open a specific file, to display
        /// search results, and so forth.
        /// </summary>
        /// <param name="args">Details about the launch request and process.</param>
        protected override void OnLaunched(LaunchActivatedEventArgs args)
        {
            Frame rootFrame = Window.Current.Content as Frame;

            // Do not repeat app initialization when the Window already has content,
            // just ensure that the window is active
            if (rootFrame == null)
            {
                // Create a Frame to act as the navigation context and navigate to the first page
                rootFrame = new Frame();

                if (args.PreviousExecutionState == ApplicationExecutionState.Terminated)
                {
                    //TODO: Load state from previously suspended application
                }

                // Place the frame in the current Window
                Window.Current.Content = rootFrame;
            }

            if (rootFrame.Content == null)
            {
                // When the navigation stack isn't restored navigate to the first page,
                // configuring the new page by passing required information as a navigation
                // parameter
                if (!rootFrame.Navigate(typeof(MainPage), args.Arguments))
                {
                    throw new Exception("Failed to create initial page");
                }
            }
            // Ensure the current window is active
            Window.Current.Activate();
        }

        /// <summary>
        /// Invoked when application execution is being suspended.  Application state is saved
        /// without knowing whether the application will be terminated or resumed with the contents
        /// of memory still intact.
        /// </summary>
        /// <param name="sender">The source of the suspend request.</param>
        /// <param name="e">Details about the suspend request.</param>
        private void OnSuspending(object sender, SuspendingEventArgs e)
        {
            var deferral = e.SuspendingOperation.GetDeferral();
            //TODO: Save application state and stop any background activity
            deferral.Complete();
        }

        public static void SaveSubscriptions()
        {

            var localSettings = Windows.Storage.ApplicationData.Current.LocalSettings;
            var dataContainer = localSettings.CreateContainer("WAZSMApiContainer", Windows.Storage.ApplicationDataCreateDisposition.Always);
            string savedData = "{\"SavedSubscriptions\": [";
            int count = App.SavedSubscriptions.Count;
            for (int i=0; i < count; i++)
            {
                var item = App.SavedSubscriptions[i];
                string jsonValue = item.ToJsonFormat();
                savedData += jsonValue;
                if (i < count - 1)
                {
                    savedData += ",";
                }
            }
            savedData += "]}";
            if (dataContainer.Values.ContainsKey("SavedSubscriptions"))
            {
                dataContainer.Values["SavedSubscriptions"] = savedData;
            }
            else
            {
                dataContainer.Values.Add("SavedSubscriptions", savedData);
            }
        }

        private void ReadSavedData()
        {
            try
            {
                var localSettings = Windows.Storage.ApplicationData.Current.LocalSettings;
                var dataContainer = localSettings.CreateContainer("WAZSMApiContainer", Windows.Storage.ApplicationDataCreateDisposition.Always);
                if (dataContainer.Values.ContainsKey("SavedSubscriptions"))
                {
                    string savedData = (string)dataContainer.Values["SavedSubscriptions"];
                    if (!string.IsNullOrWhiteSpace(savedData))
                    {
                        JsonObject obj = JsonObject.Parse(savedData);
                        var arr = obj["SavedSubscriptions"].GetArray();
                        if (arr != null && arr.Count > 0)
                        {
                            foreach (var item in arr)
                            {
                                obj = item.GetObject();
                                var keys = obj.Keys;
                                var savedSubscription = new SavedSubscription();
                                foreach (var key in keys)
                                {
                                    switch (key)
                                    {
                                        case "SubscriptionId":
                                            savedSubscription.SubscriptionId = obj[key].GetString();
                                            break;
                                        case "FriendlyName":
                                            savedSubscription.FriendlyName = obj[key].GetString();
                                            break;
                                    }
                                }
                                App.SavedSubscriptions.Add(savedSubscription);
                            }
                        }
                    }
                }
            }
            catch
            {
            }
        }
    }
}
