﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.Storage;
using Windows.Storage.Pickers;
using System.Threading.Tasks;
using Windows.Security.Cryptography.Certificates;
// The Basic Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234237

namespace WAZSMApiClient.Pages
{
    /// <summary>
    /// A basic page that provides characteristics common to most applications.
    /// </summary>
    public sealed partial class SetupSubscription : WAZSMApiClient.Common.LayoutAwarePage
    {
        public SetupSubscription()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// Populates the page with content passed during navigation.  Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="navigationParameter">The parameter value passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested.
        /// </param>
        /// <param name="pageState">A dictionary of state preserved by this page during an earlier
        /// session.  This will be null the first time a page is visited.</param>
        protected override void LoadState(Object navigationParameter, Dictionary<String, Object> pageState)
        {
        }

        /// <summary>
        /// Preserves state associated with this page in case the application is suspended or the
        /// page is discarded from the navigation cache.  Values must conform to the serialization
        /// requirements of <see cref="SuspensionManager.SessionState"/>.
        /// </summary>
        /// <param name="pageState">An empty dictionary to be populated with serializable state.</param>
        protected override void SaveState(Dictionary<String, Object> pageState)
        {
        }

        private void RadioButtonPfxFileOrPublishSetting_Click_1(object sender, RoutedEventArgs e)
        {
            var rb = sender as RadioButton;
            TextBoxSubscriptionId.IsEnabled = false;
            TextBoxSubscriptionId.Text = "";
            TextBoxSubscriptionFriendlyName.IsEnabled = false;
            TextBoxSubscriptionFriendlyName.Text = "";
            TextBoxPfxFile.IsEnabled = false;
            TextBoxPfxFile.Text = "";
            ButtonChoosePfxFile.IsEnabled = false;
            TextBoxPfxFilePassword.IsEnabled = false;
            TextBoxPfxFilePassword.Password = "";
            ButtonChoosePublishSettingsFile.IsEnabled = false;
            TextBoxPublishSettingsFile.IsEnabled = false;
            TextBoxPublishSettingsFile.Text = "";
            ButtonSave.IsEnabled = true;
            switch (rb.Name)
            {
                case "RadioButtonPfxFile":
                    TextBoxSubscriptionId.IsEnabled = true;
                    TextBoxSubscriptionFriendlyName.IsEnabled = true;
                    TextBoxPfxFile.IsEnabled = true;
                    ButtonChoosePfxFile.IsEnabled = true;
                    TextBoxPfxFilePassword.IsEnabled = true;
                    TextBoxSubscriptionId.Focus(Windows.UI.Xaml.FocusState.Keyboard);
                    break;
                case "RadioButtonPublishSettingsFile":
                    ButtonChoosePublishSettingsFile.IsEnabled = true;
                    TextBoxPublishSettingsFile.IsEnabled = true;
                    ButtonChoosePublishSettingsFile.Focus(Windows.UI.Xaml.FocusState.Programmatic);
                    break;
            }
        }

        private async void ButtonChoosePfxFile_Click_1(object sender, RoutedEventArgs e)
        {
            var file = await ChooseFile(".pfx");
            if (file != null)
            {
                selectedFile = file;
                TextBoxPfxFile.Text = file.Path;
            }
        }

        private async void ButtonChoosePublishSettingsFile_Click_1(object sender, RoutedEventArgs e)
        {
            var file = await ChooseFile(".publishsettings");
            if (file != null)
            {
                TextBoxPublishSettingsFile.Text = file.Path;
            }
        }
        StorageFile selectedFile;

        private async Task<StorageFile> ChooseFile(string fileExtension)
        {
            var picker = new FileOpenPicker();

            picker.SuggestedStartLocation = PickerLocationId.DocumentsLibrary | PickerLocationId.Downloads | PickerLocationId.ComputerFolder | PickerLocationId.Desktop;
            picker.ViewMode = PickerViewMode.Thumbnail;
            picker.FileTypeFilter.Add(fileExtension);
            var file = await picker.PickSingleFileAsync();
            return file;
        }

        private void ButtonSave_Click_1(object sender, RoutedEventArgs e)
        {
            if (RadioButtonPfxFile.IsChecked.Value)
            {
                if (ValidateEntriesForPfxFileOption())
                {
                    SaveSubscriptionInformation();
                }
            }
            else if (RadioButtonPublishSettingsFile.IsChecked.Value)
            {
                if (ValidateEntriesForPublishSettingsFileOption())
                {
                }
            }
        }

        private bool ValidateEntriesForPfxFileOption()
        {
            TextBoxSubscriptionId.Text = TextBoxSubscriptionId.Text.Trim();
            Guid subId;
            if (! Guid.TryParse(TextBoxSubscriptionId.Text, out subId))
            {
                var msgBox = new Windows.UI.Popups.MessageDialog("Invalid subscription id. Subscription id should be a GUID.").ShowAsync();
                return false;
            }
            if (string.IsNullOrWhiteSpace(TextBoxPfxFile.Text))
            {
                var msgBox = new Windows.UI.Popups.MessageDialog("Please select a valid pfx file.").ShowAsync();
                return false;
            }
            if (string.IsNullOrWhiteSpace(TextBoxPfxFilePassword.Password))
            {
                var msgBox = new Windows.UI.Popups.MessageDialog("Please specify a password for pfx file.").ShowAsync();
                return false;
            }
            if (string.IsNullOrWhiteSpace(TextBoxSubscriptionFriendlyName.Text))
            {
                TextBoxSubscriptionFriendlyName.Text = TextBoxSubscriptionId.Text;
            }
            TextBoxSubscriptionId.Text = TextBoxSubscriptionId.Text.Trim();
            return true;
        }

        private bool ValidateEntriesForPublishSettingsFileOption()
        {
            if (string.IsNullOrWhiteSpace(TextBoxPublishSettingsFile.Text))
            {
                var msgBox = new Windows.UI.Popups.MessageDialog("Please select a valid publish settings file.").ShowAsync();
                return false;
            }

            return true; ;
        }

        private async void SaveSubscriptionInformation()
        {
            try
            {
                if (App.SavedSubscriptions.FirstOrDefault(s => s.SubscriptionId == TextBoxSubscriptionId.Text) != null)
                {
                    throw new InvalidOperationException("Specified subscription already exists in the database.");
                }
                //var file = await StorageFile..GetFileFromPathAsync(TextBoxPfxFile.Text);
                var buffer = await Windows.Storage.FileIO.ReadBufferAsync(selectedFile);
                var encodedString = Windows.Security.Cryptography.CryptographicBuffer.EncodeToBase64String(buffer);
                await Windows.Security.Cryptography.Certificates.CertificateEnrollmentManager.ImportPfxDataAsync(
                    encodedString,
                    TextBoxPfxFilePassword.Password,
                    ExportOption.NotExportable,
                    KeyProtectionLevel.NoConsent,
                    InstallOptions.None,
                    "Windows Azure Tools");
                App.SavedSubscriptions.Add(new SavedSubscription()
                {
                    SubscriptionId = TextBoxSubscriptionId.Text,
                    FriendlyName = string.IsNullOrWhiteSpace(TextBoxSubscriptionFriendlyName.Text) ? TextBoxSubscriptionId.Text : TextBoxSubscriptionFriendlyName.Text,
                });
                App.SaveSubscriptions();
                var msgBox = new Windows.UI.Popups.MessageDialog("Subscription added successfully.").ShowAsync();
                Frame rootFrame = Window.Current.Content as Frame;
                rootFrame.Navigate(typeof(MainPage));
            }
            catch (Exception excep)
            {
                var msgBox = new Windows.UI.Popups.MessageDialog("Error in saving subscription information: \n" + excep.Message).ShowAsync();
            }
        }
    }
}
